#include <systime.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <msg/msg.h>
#include "mailfront.h"

static response resp;
static str rblheader;
static str rblmessage;

static int add_header_add(str* s)
{
  const char* add = session_getenv("RBLCHECK");
  if (add != 0) {
    if (!str_cats(s, "X-RBL-Check: ")) return 0;
    if (!str_cats(s, add)) return 0;
    if (!str_catc(s, '\n')) return 0;
  }
  return 1;
}

static const response* data_start(int fd)
{
  const char* rblcheck_env = session_getenv("RBLCHECK");
  const char* ip = session_getenv("TCPREMOTEIP");

  if (rblcheck_env == 0) return 0;

  /* if RELAYCLIENT is set or authenticated user then we skip rblcheck */
  if ((session_getenv("RELAYCLIENT") != 0) || session_getnum("authenticated", 0)              ) {
    //warn1("rblcheck plugin skipped: relayclient/authenticated user");
    return 0;
  } else {
    /* If the session is not authenticated */
    if ((session_getenv("RBLCHECK_REJECT") != 0) || (session_getenv("RBLCHECK_DEFER") != 0)) {
      if ((session_getenv("RBLCHECK_REJECT") != 0)) {
        resp.number = 553;
      } else {
        resp.number = 451;
      }
      if (!str_cats(&rblmessage, "Your IP ")) return 0;
      if (!str_cats(&rblmessage, ip)) return 0;
      if (!str_cats(&rblmessage, " is RBL listed at: ")) return 0;
      if (!str_cats(&rblmessage, rblcheck_env)) return 0;
      if (rblmessage.len > 0)
        resp.message = rblmessage.s;
      else
        resp.message = "Your IP is listed in one or more RBL list.";
      return &resp;
    }
  }

  rblheader.len = 0;
  if (!add_header_add(&rblheader))
    return &resp_internal;
  return backend_data_block(rblheader.s, rblheader.len);
  (void)fd;
}

struct plugin plugin = {
  .version = PLUGIN_VERSION,
  .data_start = data_start,
};
